#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Initialization of LFPy, a Python module for simulating extracellular potentials.

Group of Computational Neuroscience,
Department of Mathematical Sciences and Technology,
Norwegian University of Life Sciences.

Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

:Classes:
  * Cell - The pythonic neuron object itself laying on top of NEURON representing cells
  * TemplateCell - Similar to Cell, but for models using cell templates
  * Synapse - Convenience class for inserting synapses onto Cell objects
  * StimIntElectrode - Convenience class for inserting electrodes onto Cell objects
  * PointProcess - Parent class of Synapse and StimIntElectrode
  * RecExtElectrode - Class for performing simulations of extracellular potentials
  * RecMEAElectrode - Class for performing simulations of in vitro (slice) extracellular potentials
  * Network - Class for creating distributed populations of cells and handling connections between cells in populations
  * NetworkCell - Similar to `TemplateCell` with some attributes and methods for spike communication between parallel RANKs
  * NetworkPopulation - Class representing group of Cell objects distributed across MPI RANKs
  * OneSphereVolumeConductor - For computing extracellular potentials within and outside a homogeneous sphere
  * FourSphereVolumeConductor - For computing extracellular potentials in 4-sphere model (brain, CSF, skull, scalp)
  * InfiniteVolumeConductor - To compute extracellular potentials with current dipoles in infinite volume conductor
  * MEG - Class for computing magnetic field from current dipole moment
:Modules:
  * lfpcalc - Functions used by RecExtElectrode class
  * tools - Some convenient functions
  * inputgenerators - Functions for synaptic input time generation
  * eegcalc - Functions for calculating current dipole moment vector P and P_tot from currents and distances.
  * run_simulations - Functions to run NEURON simulations
"""

from .version import version as __version__

from .pointprocess import Synapse, PointProcess, StimIntElectrode
from .recextelectrode import RecExtElectrode, RecMEAElectrode
from .cell import Cell
from .templatecell import TemplateCell
from .network import NetworkCell, NetworkPopulation, Network
from .test import _test as run_tests
from .eegmegcalc import OneSphereVolumeConductor, FourSphereVolumeConductor, InfiniteVolumeConductor, get_current_dipole_moment, MEG
from . import lfpcalc
from . import tools
from . import inputgenerators
from . import run_simulation



#!/usr/bin/env python
'''
################################################################################
#
# This is an example scripts using LFPy with a passive cell model adapted from
# Mainen and Sejnowski, Nature 1996, for the original files, see
# http://senselab.med.yale.edu/modeldb/ShowModel.asp?model=2488
#
# Here, excitatory and inhibitory neurons are distributed on different parts of
# the morphology, with stochastic spike times produced by the
# LFPy.inputgenerators.stationary_gamma() function.
#
# Same as "example6.py", just without the active conductances
################################################################################
'''

# importing some modules, setting some matplotlib values for pl.plot.
import LFPy
import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
plt.rcParams.update({'font.size' : 12,
                     'figure.facecolor' : '1',
                     'figure.subplot.wspace' : 0.5,
                     'figure.subplot.hspace' : 0.5})

#seed for random generation
np.random.seed(1234)

################################################################################
# A couple of function declarations
################################################################################

def insert_synapses(synparams, section, n, spTimesFun, args):
    '''find n compartments to insert synapses onto'''
    idx = cell.get_rand_idx_area_norm(section=section, nidx=n)

    #Insert synapses in an iterative fashion
    for i in idx:
        synparams.update({'idx' : int(i)})

        # Some input spike train using the function call
        spiketimes = spTimesFun(args[0], args[1], args[2], args[3], args[4])
        
        # Create synapse(s) and setting times using the Synapse class in LFPy
        s = LFPy.Synapse(cell, **synparams)
        s.set_spike_times(spiketimes)


def plot_nsg_example(cell, electrode):
    '''plotting function used by example3/4'''
    fig = plt.figure(figsize=[12, 8])
    
    #plot the somatic trace
    ax = fig.add_axes([0.1, 0.7, 0.5, 0.2])
    ax.plot(cell.tvec, cell.somav)
    ax.set_xlabel('Time [ms]')
    ax.set_ylabel('Soma pot. [mV]')
    
    #plot the synaptic current
    ax = fig.add_axes([0.1, 0.4, 0.5, 0.2])
    for i in range(len(cell.synapses)):
        ax.plot(cell.tvec, cell.synapses[i].i, color=cell.synapses[i].color)
    ax.set_xlabel('Time [ms]')
    ax.set_ylabel('Syn. i [nA]')
    
    #plot the LFP as image plot
    ax = fig.add_axes([0.1, 0.1, 0.5, 0.2])
    absmaxLFP = abs(np.array([electrode.LFP.max(), electrode.LFP.min()])).max()
    im = ax.pcolormesh(cell.tvec, electrode.z, electrode.LFP,
                  vmax=absmaxLFP, vmin=-absmaxLFP,
           cmap='spectral_r')
    
    rect = np.array(ax.get_position().bounds)
    rect[0] += rect[2] + 0.01
    rect[2] = 0.02
    cax = fig.add_axes(rect)
    cbar = plt.colorbar(im, cax=cax)
    cbar.set_label('LFP (mV)')
    ax.axis(ax.axis('tight'))
    ax.set_xlabel('Time [ms]')
    ax.set_ylabel('z [$\mu$m]')
    
    #plot the morphology, electrode contacts and synapses
    ax = fig.add_axes([0.65, 0.1, 0.25, 0.8], frameon=False)
    for secname in cell.allsecnames:
        idx = cell.get_idx(secname)
        ax.plot(np.r_[cell.xstart[idx], cell.xend[idx][-1]],
                np.r_[cell.zstart[idx], cell.zend[idx][-1]],
                color='k')
    for i in range(len(cell.synapses)):
        ax.plot([cell.synapses[i].x], [cell.synapses[i].z], \
            color=cell.synapses[i].color, marker=cell.synapses[i].marker)
    for i in range(electrode.x.size):
        ax.plot(electrode.x[i], electrode.z[i], color='g', marker='o')
    plt.axis('equal')
    plt.axis(np.array(plt.axis())*0.8)
    ax.set_xticks([])
    ax.set_yticks([])
    
    return fig


################################################################################
# Define parameters, using dictionaries
# It is possible to set a few more parameters for each class or functions, but
# we chose to show only the most important ones here.
################################################################################

#define cell parameters used as input to cell-class
cellParameters = {
    'morphology' : 'L5_Mainen96_wAxon_LFPy.hoc',
    'rm' : 30000,               # membrane resistance
    'cm' : 1.0,                 # membrane capacitance
    'Ra' : 150,                 # axial resistance
    'v_init' : -65,             # initial crossmembrane potential
    'e_pas' : -65,              # reversal potential passive mechs
    'passive' : True,           # switch on passive mechs
    'nsegs_method' : 'lambda_f',# method for setting number of segments,
    'lambda_f' : 100,           # segments are isopotential at this frequency
    'dt' : 2**-4,               # dt of LFP and NEURON simulation.
    'tstart' : -100,          #start time, recorders start at t=0
    'tstop' : 200,            #stop time of simulation
    #'custom_code'  : ['active_declarations_example3.hoc'], # will run this file
}

# Synaptic parameters taken from Hendrickson et al 2011
# Excitatory synapse parameters:
synapseParameters_AMPA = {
    'e' : 0,                    #reversal potential
    'syntype' : 'Exp2Syn',      #conductance based exponential synapse
    'tau1' : 1.,                #Time constant, rise
    'tau2' : 3.,                #Time constant, decay
    'weight' : 0.005,           #Synaptic weight
    'color' : 'r',              #for plt.plot
    'marker' : '.',             #for plt.plot
    'record_current' : True,    #record synaptic currents
}
# Excitatory synapse parameters
synapseParameters_NMDA = {         
    'e' : 0,
    'syntype' : 'Exp2Syn',
    'tau1' : 10.,
    'tau2' : 30.,
    'weight' : 0.005,
    'color' : 'm',
    'marker' : '.',
    'record_current' : True,
}
# Inhibitory synapse parameters
synapseParameters_GABA_A = {         
    'e' : -80,
    'syntype' : 'Exp2Syn',
    'tau1' : 1.,
    'tau2' : 12.,
    'weight' : 0.005,
    'color' : 'b',
    'marker' : '.',
    'record_current' : True
}
# where to insert, how many, and which input statistics
insert_synapses_AMPA_args = {
    'section' : 'apic',
    'n' : 100,
    'spTimesFun' : LFPy.inputgenerators.stationary_gamma,
    'args' : [cellParameters['tstart'], cellParameters['tstop'], 0.5, 40,
              cellParameters['tstart']]
}
insert_synapses_NMDA_args = {
    'section' : ['dend', 'apic'],
    'n' : 15,
    'spTimesFun' : LFPy.inputgenerators.stationary_gamma,
    'args' : [cellParameters['tstart'], cellParameters['tstop'], 2, 50,
              cellParameters['tstart']]
}
insert_synapses_GABA_A_args = {
    'section' : 'dend',
    'n' : 100,
    'spTimesFun' : LFPy.inputgenerators.stationary_gamma,
    'args' : [cellParameters['tstart'], cellParameters['tstop'], 0.5, 40,
              cellParameters['tstart']]
}

# Define electrode geometry corresponding to a laminar electrode, where contact
# points have a radius r, surface normal vectors N, and LFP calculated as the
# average LFP in n random points on each contact:
N = np.empty((16, 3))
for i in range(N.shape[0]): N[i,] = [1, 0, 0] #normal unit vec. to contacts
# put parameters in dictionary
electrodeParameters = {
    'sigma' : 0.3,              # Extracellular potential
    'x' : np.zeros(16) + 25,      # x,y,z-coordinates of electrode contacts
    'y' : np.zeros(16),
    'z' : np.linspace(-500, 1000, 16),
    'n' : 20,
    'r' : 10,
    'N' : N,
}

# Parameters for the cell.simulate() call, recording membrane- and syn.-currents
simulationParameters = {
    'rec_imem' : True,  # Record Membrane currents during simulation
    'rec_isyn' : True,  # Record synaptic currents
}

################################################################################
# Main simulation procedure
################################################################################

#Initialize cell instance, using the LFPy.Cell class
cell = LFPy.Cell(**cellParameters)

#set cell alignment
cell.set_rotation(x=4.98919, y=-4.33261, z=0)
cell.set_pos = (0, 0, 0)

#Insert synapses using the function defined earlier
insert_synapses(synapseParameters_AMPA, **insert_synapses_AMPA_args)
insert_synapses(synapseParameters_NMDA, **insert_synapses_NMDA_args)
insert_synapses(synapseParameters_GABA_A, **insert_synapses_GABA_A_args)

#perform NEURON simulation, results saved as attributes in the cell instance
cell.simulate(**simulationParameters)

# Initialize electrode geometry, then calculate the LFP, using the
# LFPy.RecExtElectrode class. Note that now cell is given as input to electrode
# and created after the NEURON simulations are finished
electrode = LFPy.RecExtElectrode(cell, **electrodeParameters)
print('simulating LFPs....')
electrode.calc_lfp()
print('done')

#plotting some variables and geometry, saving output to .pdf.
fig = plot_nsg_example(cell, electrode)
fig.savefig('nsg_example.pdf', dpi=300)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import unittest
import numpy as np
import neuron


class testMisc(unittest.TestCase):
    """
    test misc. code
    """

    def test_neuron_record_i_membrane_methods_00(self):
        '''not a test of LFPy per se, but we're using this method for
        calculating with the i_membrane_ attribute on each time step'''
        # sections
        soma = neuron.h.Section(name='soma')
        dend = neuron.h.Section(name='dend')

        # connect sections
        dend.connect(soma, 1, 0)

        # geometry
        soma.L = 30.
        soma.diam = 30.
        soma.nseg = 1
        dend.L = 500.
        dend.diam = 2.
        dend.nseg = 50

        # biophysical parameters
        for sec in [soma, dend]:
            sec.Ra = 100
            sec.cm = 1
            sec.insert('pas')
            for seg in sec:
                seg.pas.g = 0.0002
                seg.pas.e = -65.

        # stimulus
        syn = neuron.h.ExpSyn(0.5, sec=dend)
        syn.e = 0.
        syn.tau = 2.

        # generators
        ns = neuron.h.NetStim(0.5)
        ns.noise = 1.
        ns.start = 0.
        ns.number = 1000
        ns.interval = 10.
        nc = neuron.h.NetCon(ns, syn)
        nc.weight[0] = .01

        # integrator
        cvode = neuron.h.CVode()
        cvode.use_fast_imem(1)

        # record
        i_membrane_control = []     # record currents using Vector.record method
        i_membrane_fadvance = []    # record seg.i_membrane_ at each timestep
        for sec in [soma, dend]:
            for seg in sec:
                i = neuron.h.Vector()
                i.record(seg._ref_i_membrane_)
                i_membrane_control.append(i)
                i_membrane_fadvance.append([])

        # Simulation control
        neuron.h.dt =  2**-4          # simulation time resolution
        tstop = 500.        # simulation duration
        v_init = -65.       # membrane voltage(s) at t = 0

        def initialize():
            neuron.h.finitialize(v_init)
            neuron.h.fcurrent()

        def collect_i_membrane():
            j = 0
            for sec in [soma, dend]:
                for seg in sec:
                    i_membrane_fadvance[j].append(seg.i_membrane_)
                    j += 1

        def integrate():
            while neuron.h.t < tstop:
                collect_i_membrane()
                neuron.h.fadvance()
            collect_i_membrane() # otherwise shape mismatch

        initialize()
        integrate()

        i_membrane_control = np.array(i_membrane_control)
        i_membrane_fadvance = np.array(i_membrane_fadvance)

        np.testing.assert_equal(i_membrane_control, i_membrane_fadvance)


    def test_neuron_record_i_membrane_methods_01(self):
        '''not a test of LFPy per se, but we're using this method for
        calculating with the i_membrane_ attribute on each time step'''
        # sections
        soma = neuron.h.Section(name='soma')
        dend = neuron.h.Section(name='dend')

        # connect sections
        dend.connect(soma, 1, 0)

        # geometry
        soma.L = 30.
        soma.diam = 30.
        soma.nseg = 1
        dend.L = 500.
        dend.diam = 2.
        dend.nseg = 50

        # biophysical parameters
        for sec in [soma, dend]:
            sec.Ra = 100
            sec.cm = 1
            sec.insert('pas')
            for seg in sec:
                seg.pas.g = 0.0002
                seg.pas.e = -65.

        # stimulus
        syn = neuron.h.ExpSyn(0.5, sec=dend)
        syn.e = 0.
        syn.tau = 2.

        # generators
        ns = neuron.h.NetStim(0.5)
        ns.noise = 1.
        ns.start = 0.
        ns.number = 1000
        ns.interval = 10.
        nc = neuron.h.NetCon(ns, syn)
        nc.weight[0] = .01

        # integrator
        cvode = neuron.h.CVode()
        cvode.use_fast_imem(1)

        # record
        i_membrane_control = []     # record currents using Vector.record method
        i_membrane_fadvance = []    # record seg.i_membrane_ at each timestep
        for sec in [soma, dend]:
            for seg in sec:
                i = neuron.h.Vector()
                i.record(seg._ref_i_membrane_)
                i_membrane_control.append(i)
                i_membrane_fadvance.append([])

        # Simulation control
        neuron.h.dt = 2**-4          # simulation time resolution
        tstop = 500.        # simulation duration
        v_init = -65.       # membrane voltage(s) at t = 0

        def initialize():
            neuron.h.finitialize(v_init)
            neuron.h.fcurrent()
            neuron.h.frecord_init()
            neuron.h.t = -100. # force simulations to start at some negative t

        def collect_i_membrane():
            j = 0
            for sec in [soma, dend]:
                for seg in sec:
                    i_membrane_fadvance[j].append(seg.i_membrane_)
                    j += 1

        def integrate():
            while neuron.h.t < tstop:
                collect_i_membrane()
                neuron.h.fadvance()
            collect_i_membrane() # otherwise shape mismatch

        initialize()
        integrate()

        i_membrane_control = np.array(i_membrane_control)
        i_membrane_fadvance = np.array(i_membrane_fadvance)

        np.testing.assert_equal(i_membrane_control, i_membrane_fadvance)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''Define ParameterSpace for benchmarking of scaling with different MPI pools'''
import os
import numpy as np
import operator
import pickle
import hashlib
import parameters as ps


def sort_deep_dict(d):
    '''
    sort arbitrarily deep dictionaries into tuples
    
    Arguments
    ---------
    d : dict
    
    Returns:
    x : list of tuples of tuples of tuples ...
    '''
    x = sorted(iter(d.items()), key=operator.itemgetter(0))
    for i, (key, value) in enumerate(x):
        if type(value) == dict or type(value) == ps.ParameterSet:
            y = sorted(iter(value.items()), key=operator.itemgetter(0))
            x[i] = (key, y)
            for j, (k, v) in enumerate(y):
                if type(v) == dict or type(v) == ps.ParameterSet:
                    y[j] = (k, sort_deep_dict(v))
    return x

def get_unique_id(paramset):
    '''
    create a unique hash key for input dictionary
    
    Arguments
    ---------
    paramset : dict
        parameter dictionary
    
    Returns
    -------
    key : str
        hash key
    
    '''
    sorted_params = sort_deep_dict(paramset)
    string = pickle.dumps(sorted_params)
    key = hashlib.md5(string).hexdigest()
    return key


PSPACES = dict()

# check scaling with MPI pool size
PSPACES['MPI'] = ps.ParameterSpace(dict())
PSPACES['MPI'].update(dict(
    # Population sizes
    POP_SIZE_REF = [2400, 480],
    
    # allow different seeds for different network iterations
    GLOBALSEED = ps.ParameterRange([1234, 65135, 216579876]),
    
    # MPI pool size
    MPISIZE = ps.ParameterRange([120, 240, 480, 960, 1920, 2880]),
    
    # bool flag switching LFP calculations on or off (faster)
    COMPUTE_LFP = ps.ParameterRange([False, True]),
    
    # population size scaling (multiplied with values in populationParams['POP_SIZE']):
    POPSCALING = ps.ParameterRange([1.]),
    
    # preserve expected synapse in-degree or total number of connections
    PRESERVE = ps.ParameterRange(['indegree'])
    ))

PSPACES['MPI5'] = ps.ParameterSpace(dict())
PSPACES['MPI5'].update(dict(
    # Population sizes
    POP_SIZE_REF = [12000, 2400],

    # allow different seeds for different network iterations
    GLOBALSEED = ps.ParameterRange([1234, 65135, 216579876]),
    
    # MPI pool size
    MPISIZE = ps.ParameterRange([600, 1200, 2400, 4800]),
    
    # bool flag switching LFP calculations on or off (faster)
    COMPUTE_LFP = ps.ParameterRange([False, True]),
    
    # population size scaling (multiplied with values in populationParams['POP_SIZE']):
    POPSCALING = ps.ParameterRange([1.]),
    
    # preserve expected synapse in-degree or total number of connections
    PRESERVE = ps.ParameterRange(['indegree'])
    ))



# check scaling with population size
PSPACES['POP'] = ps.ParameterSpace(dict())
PSPACES['POP'].update(dict(
    # Population sizes
    POP_SIZE_REF = [2400, 480],

    # allow different seeds for different network iterations
    GLOBALSEED = ps.ParameterRange([1234, 65135, 216579876]),
    
    # MPI pool size
    MPISIZE = ps.ParameterRange([480]),
    
    # bool flag switching LFP calculations on or off (faster)
    COMPUTE_LFP = ps.ParameterRange([False, True]),
    
    # population size scaling (multiplied with values in populationParams['POP_SIZE']):
    POPSCALING = ps.ParameterRange([0.2, 0.25, 0.5, 1., 2.0, 4.0]), 

    # preserve expected synapse in-degree or total number of connections across population scalings
    PRESERVE = ps.ParameterRange(['total', 'indegree'])
    ))

PSPACES['POP5'] = ps.ParameterSpace(dict())
PSPACES['POP5'].update(dict(
    # Population sizes
    POP_SIZE_REF = [12000, 2400],

    # allow different seeds for different network iterations
    GLOBALSEED = ps.ParameterRange([1234, 65135, 216579876]),
    
    # MPI pool size
    MPISIZE = ps.ParameterRange([2400]),
    
    # bool flag switching LFP calculations on or off (faster)
    COMPUTE_LFP = ps.ParameterRange([False, True]),
    
    # population size scaling (multiplied with values in populationParams['POP_SIZE']):
    POPSCALING = ps.ParameterRange([0.2, 0.25, 0.5, 1., 2.0, 4.0]), 

    # preserve expected synapse in-degree or total number of connections across population scalings
    PRESERVE = ps.ParameterRange(['total', 'indegree'])
    ))

# PSPACES['TEST'] = ps.ParameterSpace(dict())
# PSPACES['TEST'].update(dict(
#     # Population sizes
#     POP_SIZE_REF = [2400, 480],
# 
#     # allow different seeds for different network iterations
#     GLOBALSEED = ps.ParameterRange([1234, 123456, 1234665465, 1343645757, 12423, 982736]),
#     
#     # MPI pool size
#     MPISIZE = ps.ParameterRange([480]),
#     
#     # bool flag switching LFP calculations on or off (faster)
#     COMPUTE_LFP = ps.ParameterRange([True]),
#     
#     # population size scaling (multiplied with values in populationParams['POP_SIZE']):
#     POPSCALING = ps.ParameterRange([1.,]), 
# 
#     # preserve expected synapse in-degree or total number of connections across population scalings
#     PRESERVE = ps.ParameterRange(['indegree'])
#     ))

jobscript_stallo = '''#!/bin/bash
##################################################################
#SBATCH --job-name {}
#SBATCH --time {}
#SBATCH -o {}
#SBATCH -e {}
#SBATCH --mem-per-cpu=2000MB
#SBATCH --ntasks {}
##################################################################
# from here on we can run whatever command we want
unset DISPLAY # DISPLAY env variable somehow problematic with Slurm
srun --mpi=pmi2 python example_parallel_network.py {}
'''

jobscript_jureca = '''#!/bin/bash
##################################################################
#SBATCH --job-name {}
#SBATCH --time {}
#SBATCH -o {}
#SBATCH -e {}
#SBATCH -N {}
#SBATCH --ntasks {}
#SBATCH --mem-per-cpu={}
#SBATCH --exclusive
##################################################################
# from here on we can run whatever command we want
unset DISPLAY # DISPLAY somehow problematic with Slurm
srun python example_parallel_network.py {}
'''

LOGDIR = 'logs'
OUTPUTDIR = 'output'
JOBDIR = 'jobs'
PSETDIR = 'parameters'


if __name__ == '__main__':
    for f in [LOGDIR, JOBDIR, PSETDIR]:
        if not os.path.isdir(f):
            os.mkdir(f)
        else:
            os.system('rm {}'.format(os.path.join(f, '*')))
    
    runningjobs = []
    
    for PSPACE in PSPACES.values():    
        for pset in PSPACE.iter_inner():
            # get identifier
            ps_id = get_unique_id(pset)
            print(ps_id)
        
            # write parameterset file
            pset.save(os.path.join(PSETDIR, ps_id + '.txt'))
            
            # memory (MB)
            mem_per_cpu = 4000
            
            # walltime (2400 seconds per 480 MPI threads and popscaling 1 and neuron count 2880)
            wt = 2400 * 480 / pset.MPISIZE * pset.POPSCALING * np.sum(pset.POP_SIZE_REF) / 2880.
            wt = '%i:%.2i:%.2i' % (wt//3600, (wt-wt//3600*3600)//60, (wt-wt//60*60))
        
            # logfile
            logfile = os.path.join(LOGDIR, ps_id + '.txt')
            
            # write and submit jobscript
            try:
                if os.environ['HOSTNAME'].rfind('jr') >= 0: # JURECA
                    NPERNODE = 24
                    N = int(pset.MPISIZE / NPERNODE)
                    jobscript = jobscript_jureca.format(ps_id, wt, logfile, logfile, N, pset.MPISIZE, mem_per_cpu, ps_id)
                elif os.environ['HOSTNAME'].rfind('stallo') >= 0 or os.environ['HOSTNAME'].rfind('local') >= 0:
                    jobscript = jobscript_stallo.format(ps_id, wt, logfile, logfile, pset.MPISIZE, mem_per_cpu, ps_id)
                else:
                    pass # modify as needed, or assuming runs are done locally
                # write jobscript file
                fpath = os.path.join(JOBDIR, ps_id + '.sh')
                f = open(fpath, 'w')
                f.writelines(jobscript)
                f.close()
                
            
                # submit job
                if not ps_id in runningjobs:
                    os.system('sbatch {}'.format(fpath))
                
                runningjobs.append(ps_id)
            except KeyError:
                pass


       


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import sys
import warnings
import numpy as np
import MEAutility as mu
from copy import deepcopy
from . import lfpcalc, tools


class RecExtElectrode(object):
    """class RecExtElectrode

    Main class that represents an extracellular electric recording devices such
    as a laminar probe.

    Parameters
    ----------
    cell : None or object
        If not None, instantiation of LFPy.Cell, LFPy.TemplateCell or similar.
    sigma : float or list/ndarray of floats
        extracellular conductivity in units of (S/m). A scalar value implies an
        isotropic extracellular conductivity. If a length 3 list or array of
        floats is provided, these values corresponds to an anisotropic
        conductor with conductivities [sigma_x, sigma_y, sigma_z] accordingly.
    probe : MEAutility MEA object or None
        MEAutility probe object
    x, y, z : np.ndarray
        coordinates or arrays of coordinates in units of (um). Must be same length
    N : None or list of lists
        Normal vectors [x, y, z] of each circular electrode contact surface,
        default None
    r : float
        radius of each contact surface, default None
    n : int
        if N is not None and r > 0, the number of discrete points used to
        compute the n-point average potential on each circular contact point.
    contact_shape : str
        'circle'/'square' (default 'circle') defines the contact point shape
        If 'circle' r is the radius, if 'square' r is the side length
    method : str
        switch between the assumption of 'linesource', 'pointsource',
        'soma_as_point' to represent each compartment when computing
        extracellular potentials
    from_file : bool
        if True, load cell object from file
    cellfile : str
        path to cell pickle
    verbose : bool
        Flag for verbose output, i.e., print more information
    seedvalue : int
        random seed when finding random position on contact with r > 0

    Examples
    --------
    Compute extracellular potentials after simulating and storage of
    transmembrane currents with the LFPy.Cell class:

    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> import LFPy
    >>>
    >>> cellParameters = {
    >>>     'morphology' : 'examples/morphologies/L5_Mainen96_LFPy.hoc',  # morphology file
    >>>     'v_init' : -65,                          # initial voltage
    >>>     'cm' : 1.0,                             # membrane capacitance
    >>>     'Ra' : 150,                             # axial resistivity
    >>>     'passive' : True,                        # insert passive channels
    >>>     'passive_parameters' : {"g_pas":1./3E4, "e_pas":-65}, # passive params
    >>>     'dt' : 2**-4,                           # simulation time res
    >>>     'tstart' : 0.,                        # start t of simulation
    >>>     'tstop' : 50.,                        # end t of simulation
    >>> }
    >>> cell = LFPy.Cell(**cellParameters)
    >>>
    >>> synapseParameters = {
    >>>     'idx' : cell.get_closest_idx(x=0, y=0, z=800), # compartment
    >>>     'e' : 0,                                # reversal potential
    >>>     'syntype' : 'ExpSyn',                   # synapse type
    >>>     'tau' : 2,                              # syn. time constant
    >>>     'weight' : 0.01,                       # syn. weight
    >>>     'record_current' : True                 # syn. current record
    >>> }
    >>> synapse = LFPy.Synapse(cell, **synapseParameters)
    >>> synapse.set_spike_times(np.array([10., 15., 20., 25.]))
    >>>
    >>> cell.simulate(rec_imem=True)
    >>>
    >>> N = np.empty((16, 3))
    >>> for i in xrange(N.shape[0]): N[i,] = [1, 0, 0] #normal vec. of contacts
    >>> electrodeParameters = {         #parameters for RecExtElectrode class
    >>>     'sigma' : 0.3,              #Extracellular potential
    >>>     'x' : np.zeros(16)+25,      #Coordinates of electrode contacts
    >>>     'y' : np.zeros(16),
    >>>     'z' : np.linspace(-500,1000,16),
    >>>     'n' : 20,
    >>>     'r' : 10,
    >>>     'N' : N,
    >>> }
    >>> electrode = LFPy.RecExtElectrode(cell, **electrodeParameters)
    >>> electrode.calc_lfp()
    >>> plt.matshow(electrode.LFP)
    >>> plt.colorbar()
    >>> plt.axis('tight')
    >>> plt.show()


    Compute extracellular potentials during simulation (recommended):

    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> import LFPy
    >>>
    >>> cellParameters = {
    >>>     'morphology' : 'examples/morphologies/L5_Mainen96_LFPy.hoc',  # morphology file
    >>>     'v_init' : -65,                          # initial voltage
    >>>     'cm' : 1.0,                             # membrane capacitance
    >>>     'Ra' : 150,                             # axial resistivity
    >>>     'passive' : True,                        # insert passive channels
    >>>     'passive_parameters' : {"g_pas":1./3E4, "e_pas":-65}, # passive params
    >>>     'dt' : 2**-4,                           # simulation time res
    >>>     'tstart' : 0.,                        # start t of simulation
    >>>     'tstop' : 50.,                        # end t of simulation
    >>> }
    >>> cell = LFPy.Cell(**cellParameters)
    >>>
    >>> synapseParameters = {
    >>>     'idx' : cell.get_closest_idx(x=0, y=0, z=800), # compartment
    >>>     'e' : 0,                                # reversal potential
    >>>     'syntype' : 'ExpSyn',                   # synapse type
    >>>     'tau' : 2,                              # syn. time constant
    >>>     'weight' : 0.01,                       # syn. weight
    >>>     'record_current' : True                 # syn. current record
    >>> }
    >>> synapse = LFPy.Synapse(cell, **synapseParameters)
    >>> synapse.set_spike_times(np.array([10., 15., 20., 25.]))
    >>>
    >>> N = np.empty((16, 3))
    >>> for i in xrange(N.shape[0]): N[i,] = [1, 0, 0] #normal vec. of contacts
    >>> electrodeParameters = {         #parameters for RecExtElectrode class
    >>>     'sigma' : 0.3,              #Extracellular potential
    >>>     'x' : np.zeros(16)+25,      #Coordinates of electrode contacts
    >>>     'y' : np.zeros(16),
    >>>     'z' : np.linspace(-500,1000,16),
    >>>     'n' : 20,
    >>>     'r' : 10,
    >>>     'N' : N,
    >>> }
    >>> electrode = LFPy.RecExtElectrode(**electrodeParameters)
    >>>
    >>> cell.simulate(electrode=electrode)
    >>>
    >>> plt.matshow(electrode.LFP)
    >>> plt.colorbar()
    >>> plt.axis('tight')
    >>> plt.show()

    Use MEAutility to to handle probes

    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> import MEAutility as mu
    >>> import LFPy
    >>>
    >>> cellParameters = {
    >>>     'morphology' : 'examples/morphologies/L5_Mainen96_LFPy.hoc',  # morphology file
    >>>     'v_init' : -65,                          # initial voltage
    >>>     'cm' : 1.0,                             # membrane capacitance
    >>>     'Ra' : 150,                             # axial resistivity
    >>>     'passive' : True,                        # insert passive channels
    >>>     'passive_parameters' : {"g_pas":1./3E4, "e_pas":-65}, # passive params
    >>>     'dt' : 2**-4,                           # simulation time res
    >>>     'tstart' : 0.,                        # start t of simulation
    >>>     'tstop' : 50.,                        # end t of simulation
    >>> }
    >>> cell = LFPy.Cell(**cellParameters)
    >>>
    >>> synapseParameters = {
    >>>     'idx' : cell.get_closest_idx(x=0, y=0, z=800), # compartment
    >>>     'e' : 0,                                # reversal potential
    >>>     'syntype' : 'ExpSyn',                   # synapse type
    >>>     'tau' : 2,                              # syn. time constant
    >>>     'weight' : 0.01,                       # syn. weight
    >>>     'record_current' : True                 # syn. current record
    >>> }
    >>> synapse = LFPy.Synapse(cell, **synapseParameters)
    >>> synapse.set_spike_times(np.array([10., 15., 20., 25.]))
    >>>
    >>> cell.simulate(rec_imem=True)
    >>>
    >>> probe = mu.return_mea('Neuropixels-128')
    >>> electrode = LFPy.RecExtElectrode(cell, probe=probe)
    >>> electrode.calc_lfp()
    >>> mu.plot_mea_recording(electrode.LFP, probe)
    >>> plt.axis('tight')
    >>> plt.show()

    """

    def __init__(self, cell=None, sigma=0.3, probe=None,
                 x=None, y=None, z=None,
                 N=None, r=None, n=None, contact_shape='circle',
                 perCellLFP=False, method='linesource',
                 from_file=False, cellfile=None, verbose=False,
                 seedvalue=None, **kwargs):
        """Initialize RecExtElectrode class"""

        self.sigma = sigma
        if type(sigma) in [list, np.ndarray]:
            self.sigma = np.array(sigma)
            if not self.sigma.shape == (3,):
                raise ValueError("Conductivity, sigma, should be float "
                                 "or array of length 3: "
                                 "[sigma_x, sigma_y, sigma_z]")

            self.anisotropic = True
        else:
            self.sigma = sigma
            self.anisotropic = False

        if probe is None:
            assert x is not None and y is not None and z is not None, "Provide either a 'probe' or the " \
                                                                      "'x', 'y', and 'z'"
            if type(x) in [float, int]:
                self.x = np.array([x])
            else:
                self.x = np.array(x).flatten()
            if type(y) in [float, int]:
                self.y = np.array([y])
            else:
                self.y = np.array(y).flatten()
            if type(z) in [float, int]:
                self.z = np.array([z])
            else:
                self.z = np.array(z).flatten()
            try:
                assert ((self.x.size == self.y.size) and (self.x.size == self.z.size))
            except AssertionError:
                raise AssertionError("The number of elements in [x, y, z] must be identical")

            if N is not None:
                if type(N) != np.array:
                    try:
                        N = np.array(N)
                    except:
                        print('Keyword argument N could not be converted to a '
                              'numpy.ndarray of shape (n_contacts, 3)')
                        print(sys.exc_info()[0])
                        raise
                if N.shape[-1] == 3:
                    self.N = N
                else:
                    self.N = N.T
                    if N.shape[-1] != 3:
                        raise Exception('N.shape must be (n_contacts, 1, 3)!')
            else:
                self.N = N

            self.r = r
            self.n = n

            if contact_shape is None:
                self.contact_shape = 'circle'
            elif contact_shape in ['circle', 'square', 'rect']:
                self.contact_shape = contact_shape
            else:
                raise ValueError('The contact_shape argument must be either: '
                                 'None, \'circle\', \'square\', \'rect\'')
            if self.contact_shape == 'rect':
                assert len(np.array(self.r)) == 2, "For 'rect' shape, 'r' must indicate the 2 dimensions " \
                                                   "of the rectangle"

            positions = np.array([self.x, self.y, self.z]).T
            probe_info = {'pos': positions, 'description': 'custom', 'size': self.r, 'shape': self.contact_shape,
                          'type': 'wire', 'center': False}  # add mea type
            self.probe = mu.MEA(positions=positions, info=probe_info, normal=self.N, sigma=self.sigma)
        else:
            assert isinstance(probe, mu.core.MEA), "'probe' should be a MEAutility MEA object"
            self.probe = deepcopy(probe)
            self.x = probe.positions[:, 0]
            self.y = probe.positions[:, 1]
            self.z = probe.positions[:, 2]
            self.N = np.array([el.normal for el in self.probe.electrodes])
            self.r = self.probe.size
            self.contact_shape = self.probe.shape
            self.n = n

        self.perCellLFP = perCellLFP

        self.method = method
        self.verbose = verbose
        self.seedvalue = seedvalue

        self.kwargs = kwargs

        #None-type some attributes created by the Cell class
        self.electrodecoeff = None
        self.circle = None
        self.offsets = None

        if from_file:
            if type(cellfile) == type(str()):
                cell = tools.load(cellfile)
            elif type(cellfile) == type([]):
                cell = []
                for fil in cellfile:
                    cell.append(tools.load(fil))
            else:
                raise ValueError('cell either string or list of strings')

        if cell is not None:
            self.set_cell(cell)

        if method == 'soma_as_point':
            if self.anisotropic:
                self.lfp_method = lfpcalc.calc_lfp_soma_as_point_anisotropic
            else:
                self.lfp_method = lfpcalc.calc_lfp_soma_as_point
        elif method == 'som_as_point':
            raise RuntimeError('The method "som_as_point" is deprecated.'
                                     'Use "soma_as_point" instead')
        elif method == 'linesource':
            if self.anisotropic:
                self.lfp_method = lfpcalc.calc_lfp_linesource_anisotropic
            else:
                self.lfp_method = lfpcalc.calc_lfp_linesource
        elif method == 'pointsource':
            if self.anisotropic:
                self.lfp_method = lfpcalc.calc_lfp_pointsource_anisotropic
            else:
                self.lfp_method = lfpcalc.calc_lfp_pointsource
        else:
            raise ValueError("LFP method not recognized. "
                             "Should be 'soma_as_point', 'linesource' "
                             "or 'pointsource'")

    def set_cell(self, cell):
        """Set the supplied cell object as attribute "cell" of the
        RecExtElectrode object

        Parameters
        ----------
        cell : obj
            `LFPy.Cell` or `LFPy.TemplateCell` instance.

        Returns
        -------
        None
        """
        self.cell = cell
        if self.cell is not None:
            self.r_limit = self.cell.diam/2
            self.mapping = np.zeros((self.x.size, len(cell.xmid)))


    def _test_imem_sum(self, tolerance=1E-8):
        """Test that the membrane currents sum to zero"""
        if type(self.cell) == dict or type(self.cell) == list:
            raise DeprecationWarning('no support for more than one cell-object')

        sum_imem = self.cell.imem.sum(axis=0)
        #check if eye matrix is supplied:
        if ((self.cell.imem.shape == (self.cell.totnsegs, self.cell.totnsegs))
            and (np.all(self.cell.imem == np.eye(self.cell.totnsegs)))):
            pass
        else:
            if abs(sum_imem).max() >= tolerance:
                warnings.warn('Membrane currents do not sum to zero')
                [inds] = np.where((abs(sum_imem) >= tolerance))
                if self.cell.verbose:
                    for i in inds:
                        print('membrane current sum of celltimestep %i: %.3e'
                            % (i, sum_imem[i]))
            else:
                pass


    def calc_mapping(self, cell):
        """Creates a linear mapping of transmembrane currents of each segment
        of the supplied cell object to contribution to extracellular potential
        at each electrode contact point of the RexExtElectrode object. Sets
        the class attribute "mapping", which is a shape (n_contact, n_segs)
        ndarray, such that the extracellular potential at the contacts
        phi = np.dot(mapping, I_mem)
        where I_mem is a shape (n_segs, n_tsteps) ndarray with transmembrane
        currents for each time step of the simulation.

        Parameters
        ----------
        cell : obj
            `LFPy.Cell` or `LFPy.TemplateCell` instance.

        Returns
        -------
        mapping : ndarray
            The attribute RecExtElectrode.mapping is returned (optional)
        """
        if cell is not None:
            self.set_cell(cell)

        if self.n is not None and self.N is not None and self.r is not None:
            if self.n <= 1:
                raise ValueError("n = %i must be larger that 1" % self.n)
            else:
                pass

            self._lfp_el_pos_calc_dist()

            if self.verbose:
                print('calculations finished, %s, %s' % (str(self),
                                                         str(self.cell)))
        else:
            self._loop_over_contacts()
            if self.verbose:
                print('calculations finished, %s, %s' % (str(self),
                                                         str(self.cell)))
        # return mapping
        return self.mapping


    def calc_lfp(self, t_indices=None, cell=None):
        """Calculate LFP on electrode geometry from all cell instances.
        Will chose distributed calculated if electrode contain 'n', 'N', and 'r'

        Parameters
        ----------
        cell : obj, optional
            `LFPy.Cell` or `LFPy.TemplateCell` instance. Must be specified here
            if it was not specified at the initiation of the `RecExtElectrode`
            class
        t_indices : np.ndarray
            Array of timestep indexes where extracellular potential should
            be calculated.
        """

        self.calc_mapping(cell)

        if t_indices is not None:
            currmem = self.cell.imem[:, t_indices]
        else:
            currmem = self.cell.imem

        self._test_imem_sum()
        self.LFP = np.dot(self.mapping, currmem)
        # del self.mapping

    def _loop_over_contacts(self, **kwargs):
        """Loop over electrode contacts, and return LFPs across channels"""

        for i in range(self.x.size):
            self.mapping[i, :] = self.lfp_method(self.cell,
                                             x = self.x[i],
                                             y = self.y[i],
                                             z = self.z[i],
                                             sigma = self.sigma,
                                             r_limit = self.r_limit,
                                             **kwargs)

    def _lfp_el_pos_calc_dist(self, **kwargs):

        """
        Calc. of LFP over an n-point integral approximation over flat
        electrode surface: circle of radius r or square of side r. The
        locations of these n points on the electrode surface are random,
        within the given surface. """

        def loop_over_points(points):

            # loop over points on contact
            lfp_e = 0
            for j in range(self.n):
                tmp = self.lfp_method(self.cell,
                                      x=points[j, 0],
                                      y=points[j, 1],
                                      z=points[j, 2],
                                      r_limit=self.r_limit,
                                      sigma=self.sigma,
                                      **kwargs
                                      )

                lfp_e += tmp
                # no longer needed
                del tmp

            return lfp_e / self.n

        # extract random points for each electrode
        if self.n > 1:
            points = self.probe.get_random_points_inside(self.n)
            for i, p in enumerate(points):
                #fill in with contact average
                self.mapping[i] = loop_over_points(p)
            self.recorded_points = points
        else:
            for i, (x, y, z) in enumerate(zip(self.x, self.y, self.z)):
                self.mapping[i] = self.lfp_method(self.cell,
                                                  x=x,
                                                  y=y,
                                                  z=z,
                                                  r_limit = self.r_limit,
                                                  sigma=self.sigma,
                                                  **kwargs)
            self.recorded_points = np.array([self.x, self.y, self.z]).T


class RecMEAElectrode(RecExtElectrode):
    r"""class RecMEAElectrode

    Electrode class that represents an extracellular in vitro slice recording
    as a Microelectrode Array (MEA). Inherits RecExtElectrode class

    Set-up:

              Above neural tissue (Saline) -> sigma_S
    <----------------------------------------------------> z = z_shift + h

              Neural Tissue -> sigma_T

                   o -> source_pos = [x',y',z']

    <-----------*----------------------------------------> z = z_shift + 0
                 \-> elec_pos = [x,y,z]

              Below neural tissue (MEA Glass plate) -> sigma_G

    Parameters
    ----------
    cell : None or object
        If not None, instantiation of LFPy.Cell, LFPy.TemplateCell or similar.
    sigma_T : float
        extracellular conductivity of neural tissue in unit (S/m)
    sigma_S : float
        conductivity of saline bath that the neural slice is
        immersed in [1.5] (S/m)
    sigma_G : float
        conductivity of MEA glass electrode plate. Most commonly
        assumed non-conducting [0.0] (S/m)
    h : float, int
        Thickness in um of neural tissue layer containing current
        the current sources (i.e., in vitro slice or cortex)
    z_shift : float, int
        Height in um of neural tissue layer bottom. If e.g., top of neural
        tissue layer should be z=0, use z_shift=-h. Defaults to z_shift = 0, so
        that the neural tissue layer extends from z=0 to z=h.
    squeeze_cell_factor : float or None
        Factor to squeeze the cell in the z-direction. This is
        needed for large cells that are thicker than the slice, since no part
        of the cell is allowed to be outside the slice. The squeeze is done
        after the neural simulation, and therefore does not affect neuronal
        simulation, only calculation of extracellular potentials.
    probe : MEAutility MEA object or None
        MEAutility probe object
    x, y, z : np.ndarray
        coordinates or arrays of coordinates in units of (um).
        Must be same length
    N : None or list of lists
        Normal vectors [x, y, z] of each circular electrode contact surface,
        default None
    r : float
        radius of each contact surface, default None
    n : int
        if N is not None and r > 0, the number of discrete points used to
        compute the n-point average potential on each circular contact point.
    contact_shape : str
        'circle'/'square' (default 'circle') defines the contact point shape
        If 'circle' r is the radius, if 'square' r is the side length
    method : str
        switch between the assumption of 'linesource', 'pointsource',
        'soma_as_point' to represent each compartment when computing
        extracellular potentials
    from_file : bool
        if True, load cell object from file
    cellfile : str
        path to cell pickle
    verbose : bool
        Flag for verbose output, i.e., print more information
    seedvalue : int
        random seed when finding random position on contact with r > 0

    Examples
    See also examples/example_MEA.py

    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> import LFPy
    >>>
    >>> cellParameters = {
    >>>     'morphology' : 'examples/morphologies/L5_Mainen96_LFPy.hoc',  # morphology file
    >>>     'v_init' : -65,                          # initial voltage
    >>>     'cm' : 1.0,                             # membrane capacitance
    >>>     'Ra' : 150,                             # axial resistivity
    >>>     'passive' : True,                        # insert passive channels
    >>>     'passive_parameters' : {"g_pas":1./3E4, "e_pas":-65}, # passive params
    >>>     'dt' : 2**-4,                           # simulation time res
    >>>     'tstart' : 0.,                        # start t of simulation
    >>>     'tstop' : 50.,                        # end t of simulation
    >>> }
    >>> cell = LFPy.Cell(**cellParameters)
    >>> cell.set_rotation(x=np.pi/2, z=np.pi/2)
    >>> cell.set_pos(z=100)
    >>> synapseParameters = {
    >>>     'idx' : cell.get_closest_idx(x=800, y=0, z=100), # compartment
    >>>     'e' : 0,                                # reversal potential
    >>>     'syntype' : 'ExpSyn',                   # synapse type
    >>>     'tau' : 2,                              # syn. time constant
    >>>     'weight' : 0.01,                       # syn. weight
    >>>     'record_current' : True                 # syn. current record
    >>> }
    >>> synapse = LFPy.Synapse(cell, **synapseParameters)
    >>> synapse.set_spike_times(np.array([10., 15., 20., 25.]))
    >>>
    >>> MEA_electrode_parameters = {
    >>>     'sigma_T' : 0.3,      # extracellular conductivity
    >>>     'sigma_G' : 0.0,      # MEA glass electrode plate conductivity
    >>>     'sigma_S' : 1.5,      # Saline bath conductivity
    >>>     'x' : np.linspace(0, 1200, 16),  # electrode requires 1d vector of positions
    >>>     'y' : np.zeros(16),
    >>>     'z' : np.zeros(16),
    >>>     "method": "pointsource",
    >>>     "h": 300,
    >>>     "squeeze_cell_factor": 0.3,
    >>> }
    >>> MEA = LFPy.RecMEAElectrode(cell, **MEA_electrode_parameters)
    >>>
    >>> cell.simulate(electrode=MEA)
    >>>
    >>> plt.matshow(MEA.LFP)
    >>> plt.colorbar()
    >>> plt.axis('tight')
    >>> plt.show()
    """
    def __init__(self, cell=None, sigma_T=0.3, sigma_S=1.5, sigma_G=0.0,
                 h=300., z_shift=0., steps=20, probe=None,
                 x=np.array([0]), y=np.array([0]), z=np.array([0]),
                 N=None, r=None, n=None,
                 perCellLFP=False, method='linesource',
                 from_file=False, cellfile=None, verbose=False,
                 seedvalue=None, squeeze_cell_factor=None, **kwargs):

        RecExtElectrode.__init__(self, cell=cell,
                     x=x, y=y, z=z, probe=probe,
                     N=N, r=r, n=n,
                     perCellLFP=perCellLFP, method=method,
                     from_file=from_file, cellfile=cellfile, verbose=verbose,
                     seedvalue=seedvalue, **kwargs)

        self.sigma_G = sigma_G
        self.sigma_T = sigma_T
        self.sigma_S = sigma_S
        self.sigma = None
        self.h = h
        self.z_shift = z_shift
        self.steps = steps
        self.squeeze_cell_factor = squeeze_cell_factor
        self.moi_param_kwargs = {"h": self.h,
                                 "steps": self.steps,
                                 "sigma_G": self.sigma_G,
                                 "sigma_T": self.sigma_T,
                                 "sigma_S": self.sigma_S,
                                 }

        if cell is not None:
            self.set_cell(cell)

        if method == 'pointsource':
            self.lfp_method = lfpcalc.calc_lfp_pointsource_moi
        elif method == "linesource":
            if (np.abs(z - self.z_shift) > 1e-9).any():
                raise NotImplementedError("The method 'linesource' is only "
                                          "supported for electrodes at the "
                                          "z=0 plane. Use z=0 or method "
                                          "'pointsource'.")
            if np.abs(self.sigma_G) > 1e-9:
                raise NotImplementedError("The method 'linesource' is only "
                                          "supported for sigma_G=0. Use "
                                          "sigma_G=0 or method "
                                          "'pointsource'.")
            self.lfp_method = lfpcalc.calc_lfp_linesource_moi
        elif method == "soma_as_point":
            if (np.abs(z - self.z_shift) > 1e-9).any():
                raise NotImplementedError("The method 'soma_as_point' is only "
                                          "supported for electrodes at the "
                                          "z=0 plane. Use z=0 or method "
                                          "'pointsource'.")
            if np.abs(self.sigma_G) > 1e-9:
                raise NotImplementedError("The method 'soma_as_point' is only "
                                          "supported for sigma_G=0. Use "
                                          "sigma_G=0 or method "
                                          "'pointsource'.")
            self.lfp_method = lfpcalc.calc_lfp_soma_as_point_moi
        else:
            raise ValueError("LFP method not recognized. "
                             "Should be 'soma_as_point', 'linesource' "
                             "or 'pointsource'")

    def _squeeze_cell_in_depth_direction(self):
        """Will squeeze self.cell centered around the soma by a scaling factor,
        so that it fits inside the slice. If scaling factor is not big enough,
        a RuntimeError is raised. """

        self.cell.distort_geometry(factor=self.squeeze_cell_factor)

        if (np.max([self.cell.zstart, self.cell.zend]) > self.h + self.z_shift or
            np.min([self.cell.zstart, self.cell.zend]) < self.z_shift):
            bad_comps, reason = self._return_comp_outside_slice()
            msg = ("Compartments {} of cell ({}) has cell.{} slice. "
                   "Increase squeeze_cell_factor, move or rotate cell."
                   ).format(bad_comps, self.cell.morphology, reason)

            raise RuntimeError(msg)

    def _return_comp_outside_slice(self):
        """
        Assuming part of the cell is outside the valid region,
        i.e, not in the slice (self.z_shift < z < self.z_shift + self.h)
        this function check what array (cell.zstart or cell.zend) that is
        outside, and if it is above or below the valid region.

        Raises: RuntimeError
            If no compartment is outside valid region.

        Returns: array, str
            Numpy array with the compartments that are outside the slice,
            and a string with additional information on the problem.
        """
        zstart_above = np.where(self.cell.zstart > self.z_shift + self.h)[0]
        zend_above = np.where(self.cell.zend > self.z_shift + self.h)[0]
        zend_below = np.where(self.cell.zend < self.z_shift)[0]
        zstart_below = np.where(self.cell.zstart < self.z_shift)[0]

        if len(zstart_above) > 0:
            return zstart_above, "zstart above"
        if len(zstart_below) > 0:
            return zstart_below, "zstart below"
        if len(zend_above) > 0:
            return zend_above, "zend above"
        if len(zend_below) > 0:
            return zend_below, "zend below"
        raise RuntimeError("This function should only be called if cell"
                           "extends outside slice")

    def test_cell_extent(self):
        """
        Test if the cell is confined within the slice.
        If class argument "squeeze_cell" is True, cell is squeezed to to
        fit inside slice.

        """
        if self.cell is None:
            raise RuntimeError("Does not have cell instance.")

        if (np.max([self.cell.zstart, self.cell.zend]) > self.z_shift + self.h or
                np.min([self.cell.zstart, self.cell.zend]) < self.z_shift):

            if self.verbose:
                print("Cell extends outside slice.")

            if self.squeeze_cell_factor is not None:
                if not self.z_shift < self.cell.zmid[0] < self.z_shift + self.h:
                    raise RuntimeError("Soma position is not in slice.")
                self._squeeze_cell_in_depth_direction()
            else:
                bad_comps, reason = self._return_comp_outside_slice()
                msg = ("Compartments {} of cell ({}) has cell.{} slice "
                       "and argument squeeze_cell_factor is None."
                       ).format(bad_comps, self.cell.morphology, reason)
                raise RuntimeError(msg)
        else:
            if self.verbose:
                print("Cell position is good.")
            if self.squeeze_cell_factor is not None:
                if self.verbose:
                    print("Squeezing cell anyway.")
                self._squeeze_cell_in_depth_direction()

    def calc_mapping(self, cell):
        """Creates a linear mapping of transmembrane currents of each segment
        of the supplied cell object to contribution to extracellular potential
        at each electrode contact point of the RexExtElectrode object. Sets
        the class attribute "mapping", which is a shape (n_contact, n_segs)
        ndarray, such that the extracellular potential at the contacts
        phi = np.dot(mapping, I_mem)
        where I_mem is a shape (n_segs, n_tsteps) ndarray with transmembrane
        currents for each time step of the simulation.

        Parameters
        ----------
        cell : obj
            `LFPy.Cell` or `LFPy.TemplateCell` instance.

        Returns
        -------
        None
        """
        if cell is not None:
            self.set_cell(cell)
        self.test_cell_extent()

        # Temporarily shift coordinate system so middle layer extends
        # from z=0 to z=h
        self.z = self.z - self.z_shift
        self.cell.zstart = self.cell.zstart - self.z_shift
        self.cell.zmid = self.cell.zmid - self.z_shift
        self.cell.zend = self.cell.zend - self.z_shift

        if self.n is not None and self.N is not None and self.r is not None:
            if self.n <= 1:
                raise ValueError("n = %i must be larger that 1" % self.n)
            else:
                pass

            self._lfp_el_pos_calc_dist(**self.moi_param_kwargs)

            if self.verbose:
                print('calculations finished, %s, %s' % (str(self),
                                                         str(self.cell)))
        else:
            self._loop_over_contacts(**self.moi_param_kwargs)
            if self.verbose:
                print('calculations finished, %s, %s' % (str(self),
                                                         str(self.cell)))

        # Shift coordinate system back so middle layer extends
        # from z=z_shift to z=z_shift + h
        self.z = self.z + self.z_shift
        self.cell.zstart = self.cell.zstart + self.z_shift
        self.cell.zmid = self.cell.zmid + self.z_shift
        self.cell.zend = self.cell.zend + self.z_shift


    def calc_lfp(self, t_indices=None, cell=None):
        """Calculate LFP on electrode geometry from all cell instances.
        Will chose distributed calculated if electrode contain 'n', 'N', and 'r'

        Parameters
        ----------
        cell : obj, optional
            `LFPy.Cell` or `LFPy.TemplateCell` instance. Must be specified here
            if it was not specified at the initiation of the `RecExtElectrode`
            class
        t_indices : np.ndarray
            Array of timestep indexes where extracellular potential should
            be calculated.
        """

        self.calc_mapping(cell)

        if t_indices is not None:
            currmem = self.cell.imem[:, t_indices]
        else:
            currmem = self.cell.imem

        self._test_imem_sum()
        self.LFP = np.dot(self.mapping, currmem)
        # del self.mapping



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Example plot for LFPy: In vitro MEA slice set-up. It serves to demonstrate
class LFPy.RecMEAElectrode which incorporates discontinous extracellular
conductivity (MEA chip, slice, saline) in comparison to LFPy.RecExtElectrode
which incorporates an infinite homogeneous isotropic/anisotropic extracellular
conductivity. 

Execution:

    python example_MEA.py
    
Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""
import os
from os.path import join
import sys
if sys.version < '3':
    from urllib2 import urlopen
else:
    from urllib.request import urlopen
import zipfile
import LFPy
import numpy as np
import matplotlib.pyplot as plt


def plot_results(cell, synapse, MEA, electrode):

    time_window = [0, 50]
    syn_idx = synapse.idx

    cell_plot_idxs = [syn_idx]
    cell_plot_colors = {syn_idx: 'y'}
    num_cols = 4

    fig = plt.figure(figsize=[15, 5])
    plt.subplots_adjust(hspace=0.6, wspace=0.3, right=0.99, left=0.03, top=0.9)
    ax1 = fig.add_axes([0.05, 0.5, 0.37, 0.4], aspect=1, frameon=False,
                       xticks=[], yticks=[], title='Top view')
    ax3 = fig.add_axes([0.05, 0.1, 0.37, 0.4], aspect=1, frameon=False,
                       xticks=[], yticks=[], title='Side view')

    ax_ec = fig.add_subplot(1, num_cols, 3, xlim=time_window,
                            xlabel='ms', ylabel='$\mu$V',
                            title='Extracellular\npotential')
    ax_v = plt.subplot(1, num_cols, 4, title='Somatic potential',
                       xlabel='ms', ylabel='mV',
                       ylim=[-70, -50], xlim=time_window)

    elec_clr = lambda idx: plt.cm.nipy_spectral(1./(len(MEA.x) - 1) * idx)

    l_elec, l_syn = plot_recording_set_up(cell, ax1, ax3, MEA, elec_clr,
                                              syn_idx, cell_plot_colors)
    [ax_v.plot(cell.tvec, cell.vmem[idx, :], c=cell_plot_colors[idx], lw=2)
        for idx in cell_plot_idxs]
    for elec in range(len(MEA.x)):
        ax_ec.plot(cell.tvec, 1000 * (MEA.LFP[elec]),
                   lw=2, c=elec_clr(elec))
        ax_ec.plot(cell.tvec, 1000 * (electrode.LFP[elec]),
                   lw=3, c=elec_clr(elec), ls=":")

    l_MEA, = ax_ec.plot(0, 0, lw=1, c="k")
    l_invivo, = ax_ec.plot(0, 0, lw=2, c="k", ls=":")

    fig.legend([l_syn, l_elec, l_MEA, l_invivo], ["Synapse", "Electrode", "Slice", "In vivo"],
               frameon=False, numpoints=1, ncol=4, loc=3)
    simplify_axes([ax_v, ax_ec])
    mark_subplots([ax1, ax3, ax_ec, ax_v], ypos=1.05, xpos=-0.1)

    plt.savefig('example_MEA.pdf', dpi=150)


def plot_recording_set_up(cell, ax_neur, ax_side, MEA, elec_clr,
                           syn_idx, cell_plot_colors):

    for comp in range(len(cell.xmid)):
        if comp == 0:
            ax_neur.scatter(cell.xmid[comp], cell.ymid[comp], s=cell.diam[comp],
                            edgecolor='none', color='gray', zorder=1)
        else:
            ax_neur.plot([cell.xstart[comp], cell.xend[comp]],
                         [cell.ystart[comp], cell.yend[comp]],
                         lw=cell.diam[comp]/2, color='gray', zorder=1)

    for comp in range(len(cell.xmid)):
        if comp == 0:
            ax_side.scatter(cell.xmid[comp], cell.zmid[comp], s=cell.diam[comp],
                            edgecolor='none', color='gray', zorder=1)
        else:
            ax_side.plot([cell.xstart[comp], cell.xend[comp]],
                         [cell.zstart[comp], cell.zend[comp]],
                         lw=cell.diam[comp]/2, color='gray', zorder=1)
    for idx in range(len(MEA.x)):
        ax_side.plot(MEA.x[idx], MEA.z[idx] - 10, 's', clip_on=False,
                     c=elec_clr(idx), zorder=10, mec='none')
        ax_side.plot([MEA.x[idx], MEA.x[idx]], [MEA.z[idx] - 10, MEA.z[idx] - 150],
                     c=elec_clr(idx), zorder=10, lw=2)
        ax_neur.plot(MEA.x[idx], MEA.y[idx], 's', c=elec_clr(idx), zorder=10)

    ax_side.axhspan(-250, 0, facecolor='0.5', edgecolor='none')
    ax_side.axhspan(0, MEA.h, facecolor='lightsalmon', edgecolor='none')
    ax_side.axhspan(MEA.h, MEA.h + 250, facecolor='aqua', edgecolor='none')
    ax_neur.axhspan(-500, 500, facecolor='lightsalmon', edgecolor='none')

    l_elec, = ax_neur.plot(MEA.x[0], MEA.y[0], 's', c=elec_clr(0), zorder=0)

    l_syn, = ax_neur.plot(cell.xmid[syn_idx], cell.ymid[syn_idx], '*',
                           c=cell_plot_colors[syn_idx], ms=15)

    ax_side.plot(cell.xmid[syn_idx], cell.zmid[syn_idx], '*',
                 c=cell_plot_colors[syn_idx], ms=15)

    ax_neur.arrow(-220, -100, 30, 0, lw=1, head_width=12, color='k', clip_on=False)
    ax_neur.arrow(-220, -100, 0, 30, lw=1, head_width=12, color='k', clip_on=False)
    ax_neur.text(-150, -100, 'x', size=10, ha='center', va='center', clip_on=False)
    ax_neur.text(-220, -20, 'y', size=10, ha='center', va='center', clip_on=False)

    ax_side.arrow(-220, 20, 30, 0, lw=1, head_width=12, color='k', clip_on=False)
    ax_side.text(-140, 25, 'x', size=10, ha='center', va='center')
    ax_side.arrow(-220, 20, 0, 30, lw=1, head_width=12, color='k', clip_on=False)
    ax_side.text(-220, 100, 'z', size=10, ha='center', va='center')

    ax_side.plot([-500, 1250], [MEA.h, MEA.h], color='k')
    ax_side.plot([-500, 1250], [0, 0], 'k')  # PLOTTING BOTTOM OF MEA

    ax_side.plot([1280, 1280], [0, MEA.h], '_-',
                 color='k', lw=2, clip_on=False, solid_capstyle='butt')
    ax_side.text(1300, MEA.h / 2, '%g $\mu$m' % MEA.h, size=8, va='center')

    ax_side.text(1200, -10, 'MEA', va='top', ha='right')
    ax_side.text(1200, MEA.h, 'Saline', va='bottom', ha='right')
    ax_side.text(1200, MEA.h - 50, 'Tissue', va='top', ha='right')

    ax_side.axis([-300, 1250, -70, MEA.h + 70])
    ax_neur.axis([-300, 1250, -250, 250])

    return l_elec, l_syn


def simplify_axes(axes):
    """
    Right and top axis line is removed from axes or list of axes

    """
    if not type(axes) is list:
        axes = [axes]

    for ax in axes:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        ax.get_xaxis().tick_bottom()
        ax.get_yaxis().tick_left()


def mark_subplots(axes, letters='ABCDEFGHIJKLMNOPQRSTUVWXYZ', xpos=-0.12, ypos=1.15):
    """ Marks subplots in axes (should be list or axes object) with capital letters
    """
    if not type(axes) is list:
        axes = [axes]

    for idx, ax in enumerate(axes):
        ax.text(xpos, ypos, letters[idx].capitalize(),
                horizontalalignment='center',
                verticalalignment='center',
                fontweight='demibold',
                fontsize=12,
                transform=ax.transAxes)


#Fetch Mainen&Sejnowski 1996 model files
if not os.path.isfile(join('cells', 'cells', 'j4a.hoc')):
    #get the model files:
    u = urlopen('http://senselab.med.yale.edu/ModelDB/eavBinDown.asp?o=2488&a=23&mime=application/zip')
    localFile = open('patdemo.zip', 'wb')
    localFile.write(u.read())
    localFile.close()
    #unzip:
    myzip = zipfile.ZipFile('patdemo.zip', 'r')
    myzip.extractall('.')
    myzip.close()

################################################################################
# Main script, set parameters and create cell, synapse and electrode objects
################################################################################

# Define cell parameters
cell_parameters = {
    'morphology' : join('cells', 'cells', 'j4a.hoc'), # from Mainen & Sejnowski, J Comput Neurosci, 1996
    'cm' : 1.0,         # membrane capacitance
    'Ra' : 150.,        # axial resistance
    'v_init' : -65.,    # initial crossmembrane potential
    'passive' : True,   # turn on NEURONs passive mechanism for all sections
    'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
    'nsegs_method' : 'lambda_f', # spatial discretization method
    'lambda_f' : 10.,           # frequency where length constants are computed
    'dt' : 2.**-4,      # simulation time step size
    'tstart' : 0.,      # start time of simulation, recorders start at t=0
    'tstop' : 50.,     # stop simulation at 100 ms.
}

# Create cell
cell = LFPy.Cell(**cell_parameters)
# Align cell
cell.set_rotation(z=np.pi/0.9, x=0.3)
cell.set_pos(z=100)

# Define synapse parameters
synapse_parameters = {
    'idx' : cell.get_closest_idx(x=800., y=0., z=150.),
    'e' : 0.,                   # reversal potential
    'syntype' : 'ExpSyn',       # synapse type
    'tau' : 5.,                 # synaptic time constant
    'weight' : .001,            # synaptic weight
    'record_current' : True,    # record synapse current
}

# Create synapse and set time of synaptic input
synapse = LFPy.Synapse(cell, **synapse_parameters)
synapse.set_spike_times(np.array([10.]))

# Create a grid of measurement locations, in (mum)
X = np.arange(0, 1001, 500)
Y = np.zeros(X.shape)
Z = np.zeros(X.shape) + 0

# Define electrode parameters
grid_electrode_parameters = {
    'sigma' : 0.3,      # extracellular conductivity
    'x' : X.flatten(),  # electrode requires 1d vector of positions
    'y' : Y.flatten(),
    'z' : Z.flatten(),
    "method": "soma_as_point",
    'N' : np.array([[0, 0, 1]]*X.size), #surface normals
    'r' : 50,              # contact site radius
    'n' : 100,               # datapoints for averaging
    "seedvalue": 12,
}

# Define electrode parameters
MEA_electrode_parameters = {
    'sigma_T' : 0.3,      # extracellular conductivity
    'sigma_G' : 0.0,      # MEA glass electrode plate conductivity
    'sigma_S' : 1.5,      # Saline bath conductivity
    'x' : X.flatten(),  # electrode requires 1d vector of positions
    'y' : Y.flatten(),
    'z' : Z.flatten(),
    "method": "soma_as_point",
    'N' : np.array([[0, 0, 1]]*X.size), #surface normals
    'r' : 50,              # contact site radius
    'n' : 100,               # datapoints for averaging,
    'h': 300,
    "seedvalue": 12,
    "squeeze_cell_factor": 0.6,
}


# Run simulation, electrode object argument in cell.simulate
print("running simulation...")
cell.simulate(rec_imem=True, rec_vmem=True)


# Create electrode objects
electrode = LFPy.RecExtElectrode(cell, **grid_electrode_parameters)
MEA = LFPy.RecMEAElectrode(cell, **MEA_electrode_parameters)

# Calculate LFPs
MEA.calc_lfp()
electrode.calc_lfp()
plot_results(cell, synapse, MEA, electrode)
plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Copyright (C) 2012 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

"""


import sys
import os
import posixpath
import unittest
import numpy as np
import LFPy
import neuron
import pickle

# for nosetests to run load the SinSyn sinusoid synapse currrent mechanism
if "win32" in sys.platform:
    pth = os.path.join(LFPy.__path__[0], 'test', 'nrnmech.dll')
    pth = pth.replace(os.sep, posixpath.sep)
    if not pth in neuron.nrn_dll_loaded:
        neuron.h.nrn_load_dll(pth)
        neuron.nrn_dll_loaded.append(pth)
else:
    neuron.load_mechanisms(os.path.join(LFPy.__path__[0], 'test'))


class testTemplateCell(unittest.TestCase):
    """
    test class LFPy.TemplateCell
    """
    def test_cell_tvec_00(self):
        stickParams = {
            'dt' : 2**-3,
            'tstart' : 0.,
            'tstop' : 100.,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_01(self):
        stickParams = {
            'dt' : 2**-3,
            'tstart' : 0.,
            'tstop' : 10000.,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_02(self):
        stickParams = {
            'dt' : 0.1,
            'tstart' : 0,
            'tstop' : 100,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_03(self):
        stickParams = {
            'dt' : 0.1,
            'tstart' : 0,
            'tstop' : 10000,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_04(self):
        stickParams = {
            'dt' : 0.2,
            'tstart' : 0,
            'tstop' : 100,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_05(self):
        stickParams = {
            'dt' : 0.2,
            'tstart' : 0,
            'tstop' : 10000,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_06(self):
        stickParams = {
            'dt' : 2**-3,
            'tstart' : -100,
            'tstop' : 100,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_07(self):
        stickParams = {
            'dt' : 2**-3,
            'tstart' : -100,
            'tstop' : 10000,
        }

        tvec = stickSimulationTesttvec(**stickParams)
        tvec_numpy = np.linspace(0, stickParams['tstop'],
                    int(stickParams['tstop']/stickParams['dt']) + 1)

        np.testing.assert_allclose(tvec, tvec_numpy, atol=1E-9)

    def test_cell_tvec_08(self):
        stickParams = {
            'dt' : 0.1,
            'tstart' : -100,
            'tstop' : 10000,
        }

        try:
            stickSimulationTesttvec(**stickParams)
        except AssertionError:
            pass

    def test_cell_tvec_09(self):
        stickParams = {
            'dt' : 0.2,
            'tstart' : -100,
            'tstop' : 10000,
        }

        try:
            stickSimulationTesttvec(**stickParams)
        except AssertionError:
            pass

    def test_cell_set_pos_00(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                         )
        np.testing.assert_allclose(cell.somapos, [0, 0, 0])

    def test_cell_set_pos_01(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        cell.set_pos(10., 20., -30.)
        np.testing.assert_allclose(cell.somapos, [10., 20., -30.])

    def test_cell_set_pos_02(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                          pt3d=True)
        cell.set_pos(10., 20., -30.)
        np.testing.assert_allclose(cell.somapos, [10., 20., -30.])

    def test_cell_set_pos_03(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                         )
        cell.set_pos(10., 20., -30.)
        cell.set_pos(10., 20., -30.)
        cell.set_pos(10., 20., -30.)
        np.testing.assert_allclose(cell.somapos, [10., 20., -30.])

    def test_cell_set_pos_04(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc'),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        pt3d=True)
        cell.set_pos(10., 20., -30.)
        cell.set_pos(10., 20., -30.)
        cell.set_pos(10., 20., -30.)
        np.testing.assert_allclose(cell.somapos, [10., 20., -30.])


    def test_cell_set_pos_05(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        np.testing.assert_allclose(cell.somapos,
                                   [cell.xmid[0], cell.ymid[0], cell.zmid[0]])


    def test_cell_set_pos_06(self):
        '''test LFPy.TemplateCell.set_pos'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        pt3d=True)
        np.testing.assert_allclose(cell.somapos,
                                   [cell.xmid[0], cell.ymid[0], cell.zmid[0]])


    def test_cell_set_rotation_00(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )

        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around x-axis
        cell.set_rotation(x=np.pi)
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.ystart, -ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, -ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, -yends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, -zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, -zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, -zends, atol=1e-07)


    def test_cell_set_rotation_01(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around y-axis
        cell.set_rotation(y=np.pi)
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, -xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, -xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, -xends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, -zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, -zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, -zends, atol=1e-07)


    def test_cell_set_rotation_02(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        # test rotation 180 deg around z-axis
        cell.set_rotation(z=np.pi)
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, -xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, -xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, -xends, atol=1e-07)
        np.testing.assert_allclose(cell.ystart, -ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, -ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, -yends, atol=1e-07)

    def test_cell_set_rotation_03(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        pt3d=True)

        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around x-axis
        cell.set_rotation(x=np.pi)
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.ystart, -ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, -ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, -yends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, -zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, -zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, -zends, atol=1e-07)


    def test_cell_set_rotation_04(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        pt3d=True)

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around y-axis
        cell.set_rotation(y=np.pi)
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, -xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, -xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, -xends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, -zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, -zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, -zends, atol=1e-07)


    def test_cell_set_rotation_05(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        pt3d=True)

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        # test rotation 180 deg around z-axis
        cell.set_rotation(z=np.pi)
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, -xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, -xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, -xends, atol=1e-07)
        np.testing.assert_allclose(cell.ystart, -ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, -ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, -yends, atol=1e-07)

    def test_cell_set_rotation_06(self):
        '''test LFPy.TemplateCell.set_rotation()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation: 90 deg around x-axis, 90 deg around y-axis, 90 deg around z-axis
        cell.set_rotation(x=np.pi / 2., y=np.pi, z=np.pi / 4.)
        # revert rotation: -90 deg around x-axis, -90 deg around y-axis, -90 deg around z-axis, rotation_order='zyx'
        cell.set_rotation(x=-np.pi / 2., y=-np.pi, z=-np.pi / 4., rotation_order='zyx')
        # assert that x-, y- and z-coordinates are same as beginning, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, xends, atol=1e-07)
        np.testing.assert_allclose(cell.ystart, ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, yends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, zends, atol=1e-07)

    def test_cell_chiral_morphology_00(self):
        '''test LFPy.TemplateCell.chiral_morphology()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                         )

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around x-axis
        cell.chiral_morphology(axis='x')
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, -xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, -xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, -xends, atol=1e-07)
        np.testing.assert_allclose(cell.ystart, ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, yends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, zends, atol=1e-07)


    def test_cell_chiral_morphology_01(self):
        '''test LFPy.TemplateCell.chiral_morphology()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around y-axis
        cell.chiral_morphology(axis='y')
        # assert that y- and z-coordinates are inverted, using absolute
        # tolerances
        np.testing.assert_allclose(cell.xstart, xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, xends, atol=1e-07)
        np.testing.assert_allclose(cell.ystart, -ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, -ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, -yends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, zends, atol=1e-07)

    def test_cell_chiral_morphology_02(self):
        '''test LFPy.TemplateCell.chiral_morphology()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )

        xstarts = cell.xstart.copy()
        xmids = cell.xmid.copy()
        xends = cell.xend.copy()
        ystarts = cell.ystart.copy()
        ymids = cell.ymid.copy()
        yends = cell.yend.copy()
        zstarts = cell.zstart.copy()
        zmids = cell.zmid.copy()
        zends = cell.zend.copy()
        # test rotation 180 deg around z-axis
        cell.chiral_morphology(axis='z')
        # assert that y- and z-coordinates are inverted, using absolute

        # tolerances
        np.testing.assert_allclose(cell.xstart, xstarts, atol=1e-07)
        np.testing.assert_allclose(cell.xmid, xmids, atol=1e-07)
        np.testing.assert_allclose(cell.xend, xends, atol=1e-07)
        np.testing.assert_allclose(cell.ystart, ystarts, atol=1e-07)
        np.testing.assert_allclose(cell.ymid, ymids, atol=1e-07)
        np.testing.assert_allclose(cell.yend, yends, atol=1e-07)
        np.testing.assert_allclose(cell.zstart, -zstarts, atol=1e-07)
        np.testing.assert_allclose(cell.zmid, -zmids, atol=1e-07)
        np.testing.assert_allclose(cell.zend, -zends, atol=1e-07)


    def test_cell_get_rand_prob_area_norm_00(self):
        '''test LFPy.TemplateCell.get_rand_prob_area_norm()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                 'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        p = cell.get_rand_prob_area_norm()
        self.assertAlmostEqual(p.sum(), 1.)
        self.assertTrue(p.min() >= 0.)
        self.assertTrue(p.max() <= 1.)


    def test_cell_get_rand_prob_area_norm_from_idx(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                 'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        p = cell.get_rand_prob_area_norm_from_idx(idx=cell.get_idx(section='allsec'))
        self.assertListEqual(cell.get_rand_prob_area_norm().tolist(), p.tolist())


    def test_cell_get_rand_prob_area_norm_from_idx_00(self):
        '''test LFPy.TemplateCell.get_rand_prob_area_norm()'''
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        p = cell.get_rand_prob_area_norm_from_idx(idx=np.array([0]))
        np.testing.assert_equal(p, np.array([1.]))


    def test_cell_get_intersegment_vector_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        idx0 = 0
        idx1 = 1
        vector = cell.get_intersegment_vector(idx0=idx0, idx1=idx1)

        self.assertListEqual(vector,
                            [cell.xmid[idx1] - cell.xmid[idx0],
                             cell.ymid[idx1] - cell.ymid[idx0],
                             cell.zmid[idx1] - cell.zmid[idx0]])


    def test_cell_get_intersegment_distance_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        idx0 = 0
        idx1 = 1
        distance = cell.get_intersegment_distance(idx0=idx0, idx1=idx1)
        vector = cell.get_intersegment_vector(idx0=idx0, idx1=idx1)

        self.assertEqual(np.sqrt(np.array(vector)**2).sum(), distance)


    def test_cell_get_idx_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        nsegs_method=None)
        self.assertListEqual(cell.get_idx(section='soma').tolist(), [0])
        self.assertListEqual(cell.get_idx(section='soma[0]').tolist(), [0])
        self.assertListEqual(cell.get_idx(section='dend[0]').tolist(), [1])
        self.assertListEqual(cell.get_idx(section='dend[1]').tolist(), [2])
        self.assertListEqual(cell.get_idx(section='dend[2]').tolist(), [3])
        self.assertListEqual(cell.get_idx(section='dend').tolist(), [1, 2, 3])
        self.assertListEqual(cell.get_idx(section='allsec').tolist(),
                             [0, 1, 2, 3])
        self.assertListEqual(cell.get_idx(section=['soma', 'dend']).tolist(),
                             [0, 1, 2, 3])
        self.assertListEqual(cell.get_idx(section='apic').tolist(), [])


    def test_cell_get_closest_idx_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        nsegs_method=None)
        self.assertEqual(cell.get_closest_idx(x=0, y=0, z=0),
                             cell.get_idx(section='soma')[0])

        self.assertEqual(cell.get_closest_idx(x=-25, y=0, z=175),
                             cell.get_idx(section='dend[1]')[0])


    def test_cell_get_idx_children_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )

        np.testing.assert_array_equal(cell.get_idx_children(parent='soma[0]'),
                                      cell.get_idx(section='dend[0]'))


    def test_cell_get_idx_parent_children_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        np.testing.assert_array_equal(cell.get_idx_parent_children(parent='soma[0]'),
                                      cell.get_idx(section=['soma[0]',
                                                            'dend[0]']))


    def test_cell_get_idx_name_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        np.testing.assert_array_equal(cell.get_idx_name(idx=np.array([0])),
                                                np.array([[0, 'ball_and_stick_template[0].soma[0]', 0.5]],
                                                         dtype=object))


    def test_cell_get_rand_idx_area_norm_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        idx = cell.get_rand_idx_area_norm(nidx=1000000)


        # compute histogram and correlate with segment area
        bins = np.arange(cell.totnsegs+1)
        hist, bin_edges = np.histogram(idx, bins=bins)

        # compute Pearson correlation coefficients between area and histogram
        # reporting success if within 5 decimal places
        self.assertAlmostEqual(np.corrcoef(cell.area, hist)[0, 1], 1., places=5)

        # check if min and max is in the range of segment indices
        self.assertEqual(idx.min(), 0)
        self.assertEqual(idx.max(), cell.totnsegs-1)


    def test_cell_set_synapse_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        cell.set_synapse(idx=0, syntype='ExpSyn', record_curret=False,
                         record_potential=False, weight=1.,
                         **dict(e=10., tau=2.))

        self.assertTrue('ExpSyn' in cell.synlist[0].hname())
        self.assertEqual(len(cell.synlist), 1)
        self.assertEqual(len(cell.netconlist), 1)
        self.assertEqual(len(cell.netstimlist), 1)
        self.assertEqual(cell.synlist[0].e, 10.)
        self.assertEqual(cell.synlist[0].tau, 2.)
        self.assertEqual(cell.netconlist[0].weight[0], 1.)


    def test_cell_set_point_process_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        cell.set_point_process(idx=0, pptype='IClamp', record_current=False,
                               **dict(delay=1., amp=1.))
        self.assertEqual(cell.stimlist[0].hname(), 'IClamp[0]')
        self.assertEqual(len(cell.stimlist), 1)
        self.assertEqual(cell.stimlist[0].delay, 1.)
        self.assertEqual(cell.stimlist[0].amp, 1.)


    def test_cell_strip_hoc_objects_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        cell.strip_hoc_objects()
        for attribute in dir(cell):
            self.assertNotEqual(str(type(getattr(cell, attribute))),
                                'hoc.HocObject')

    def test_cell_cellpickler_00(self):
        cell = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        cell_pickle = cell.cellpickler(filename=None, pickler=pickle.dumps)
        pickled_cell = pickle.loads(cell_pickle)

        for attribute in dir(cell):
            if attribute.startswith('__') or attribute.startswith('_'):
                pass
            else:
                self.assertEqual(type(getattr(cell, attribute)),
                                 type(getattr(pickled_cell, attribute)))

    def test_cell_simulate_recorder_00(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }

        stick = LFPy.TemplateCell(**stickParams)
        stick.simulate(rec_vmem=True, rec_imem=True,
                       rec_current_dipole_moment=True)
        self.assertTrue(stick.tvec.size ==
                        stick.vmem.shape[1] ==
                        stick.imem.shape[1] ==
                        stick.current_dipole_moment.shape[0])
        self.assertTrue(np.all(stick.vmem == stick.v_init))
        self.assertTrue(np.all(stick.imem == 0.))
        self.assertTrue(np.all(stick.current_dipole_moment == 0.))


    def test_cell_simulate_recorder_01(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 0.1,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }
        stick = LFPy.TemplateCell(**stickParams)
        stick.simulate(rec_vmem=True, rec_imem=True,
                       rec_current_dipole_moment=True)
        self.assertTrue(stick.tvec.size ==
                        stick.vmem.shape[1] ==
                        stick.imem.shape[1] ==
                        stick.current_dipole_moment.shape[0])
        self.assertTrue(np.all(stick.vmem == stick.v_init))
        self.assertTrue(np.all(stick.imem == 0.))
        self.assertTrue(np.all(stick.current_dipole_moment == 0.))

    def test_cell_simulate_recorder_02(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 0.2,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }
        stick = LFPy.TemplateCell(**stickParams)
        stick.simulate(rec_vmem=True, rec_imem=True,
                       rec_current_dipole_moment=True)
        self.assertTrue(stick.tvec.size ==
                        stick.vmem.shape[1] ==
                        stick.imem.shape[1] ==
                        stick.current_dipole_moment.shape[0])
        self.assertTrue(np.all(stick.vmem == stick.v_init))
        self.assertTrue(np.all(stick.imem == 0.))
        self.assertTrue(np.all(stick.current_dipole_moment == 0.))

    def test_cell_simulate_recorder_03(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 10000,
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }

        stick = LFPy.TemplateCell(**stickParams)
        stick.simulate(rec_vmem=True, rec_imem=True,
                       rec_current_dipole_moment=True)
        self.assertTrue(stick.tvec.size ==
                        stick.vmem.shape[1] ==
                        stick.imem.shape[1] ==
                        stick.current_dipole_moment.shape[0])
        self.assertTrue(np.all(stick.vmem == stick.v_init))
        self.assertTrue(np.all(stick.imem == 0.))
        self.assertTrue(np.all(stick.current_dipole_moment == 0.))


    def test_cell_simulate_recorder_04(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 10000,
            'dt' : 0.1,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }
        stick = LFPy.TemplateCell(**stickParams)
        stick.simulate(rec_vmem=True, rec_imem=True,
                       rec_current_dipole_moment=True)
        self.assertTrue(stick.tvec.size ==
                        stick.vmem.shape[1] ==
                        stick.imem.shape[1] ==
                        stick.current_dipole_moment.shape[0])
        self.assertTrue(np.all(stick.vmem == stick.v_init))
        self.assertTrue(np.all(stick.imem == 0.))
        self.assertTrue(np.all(stick.current_dipole_moment == 0.))

    def test_cell_simulate_recorder_05(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 10000,
            'dt' : 0.2,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }
        stick = LFPy.TemplateCell(**stickParams)
        stick.simulate(rec_vmem=True, rec_imem=True,
                       rec_current_dipole_moment=True)
        self.assertTrue(stick.tvec.size ==
                        stick.vmem.shape[1] ==
                        stick.imem.shape[1] ==
                        stick.current_dipole_moment.shape[0])
        self.assertTrue(np.all(stick.vmem == stick.v_init))
        self.assertTrue(np.all(stick.imem == 0.))
        self.assertTrue(np.all(stick.current_dipole_moment == 0.))

    def test_cell_simulate_current_dipole_moment_00(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 0.1,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'delay' : 0.,
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'phase' : 0,
            'bias' : 0.,
            'record_current' : False
        }

        for idx in range(31): #31 segments
            if idx != 15: # no net dipole moment because of stick symmetry
                stick = LFPy.TemplateCell(**stickParams)
                synapse = LFPy.StimIntElectrode(stick, idx=idx,
                                       **stimParams)
                stick.simulate(rec_imem=True, rec_current_dipole_moment=True)
                p = np.dot(stick.imem.T, np.c_[stick.xmid, stick.ymid, stick.zmid])
                np.testing.assert_allclose(p, stick.current_dipole_moment)


    def test_cell_simulate_current_dipole_moment_01(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : -100,
            'tstop' : 100,
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'delay' : 0.,
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'phase' : 0,
            'bias' : 0.,
            'record_current' : False
        }

        for idx in range(31): #31 segments
            if idx != 15: # no net dipole moment because of stick symmetry
                # neuron.h('forall delete_section()')
                stick = LFPy.TemplateCell(**stickParams)
                synapse = LFPy.StimIntElectrode(stick, idx=idx,
                                                **stimParams)
                stick.simulate(rec_imem=True, rec_current_dipole_moment=True)
                p = np.dot(stick.imem.T, np.c_[stick.xmid, stick.ymid, stick.zmid])
                np.testing.assert_allclose(p, stick.current_dipole_moment)

    def test_cell_simulate_current_dipole_moment_02(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : -100,
            'tstop' : 100,
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }

        stimParams = {
            'e' : 0,                                # reversal potential
            'syntype' : 'Exp2Syn',                   # synapse type
            'tau1' : 0.1,                              # syn. time constant
            'tau2' : 2.,                              # syn. time constant
            'weight' : 0.01,
        }

        for idx in range(31): #31 segments
            if idx != 15: # no net dipole moment because of stick symmetry
                stick = LFPy.TemplateCell(**stickParams)
                synapse = LFPy.Synapse(stick, idx=idx,
                                       **stimParams)
                synapse.set_spike_times(np.array([10., 20., 30., 40., 50.]))
                stick.simulate(rec_imem=True, rec_current_dipole_moment=True)
                p = np.dot(stick.imem.T, np.c_[stick.xmid, stick.ymid, stick.zmid])
                np.testing.assert_allclose(p, stick.current_dipole_moment)

    def test_cell_tstart_00(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'bias' : 0.,
            'record_current' : False
        }

        stick0 = LFPy.TemplateCell(tstart=0, tstop=200, **stickParams)
        synapse0 = LFPy.StimIntElectrode(stick0,
                                         stick0.get_closest_idx(0, 0, 1000),
                                         delay=0, phase=0.,
                                         **stimParams)
        stick0.simulate(rec_imem=True, rec_vmem=True, rec_current_dipole_moment=True)


        stick1 = LFPy.TemplateCell(tstart=-100, tstop=100, **stickParams)
        synapse1 = LFPy.StimIntElectrode(stick1,
                                         stick1.get_closest_idx(0, 0, 1000),
                                         delay=-100, phase=0.,
                                         **stimParams)
        stick1.simulate(rec_imem=True, rec_vmem=True, rec_current_dipole_moment=True)

        inds = stick0.tvec >= 100
        np.testing.assert_allclose(stick0.vmem[:, inds], stick1.vmem)
        np.testing.assert_allclose(stick0.imem[:, inds], stick1.imem)
        np.testing.assert_allclose(stick0.current_dipole_moment[inds, :],
                                   stick1.current_dipole_moment)


    def test_cell_with_recextelectrode_00(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,

        }

        electrodeParams = {
            'sigma' : 0.3,
            'x' : np.ones(11) * 100.,
            'y' : np.zeros(11),
            'z' : np.linspace(1000, 0, 11),
            'method' : 'pointsource'
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'delay' : 0.,
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'phase' : 0,
            'bias' : 0.,
            'record_current' : False
        }

        stick = LFPy.TemplateCell(**stickParams)
        synapse = LFPy.StimIntElectrode(stick, stick.get_closest_idx(0, 0, 1000),
                               **stimParams)
        electrode = LFPy.RecExtElectrode(**electrodeParams)
        stick.simulate(electrode, rec_imem=True)

        electrode1 = LFPy.RecExtElectrode(cell=stick, **electrodeParams)
        electrode1.calc_lfp()

        np.testing.assert_allclose(electrode.LFP, electrode1.LFP)
        self.assertTrue(stick.tvec.size == stick.imem.shape[1] ==
                        electrode.LFP.shape[1] == electrode1.LFP.shape[1] ==
                        int(stick.tstop/stick.dt)+1)


    def test_cell_with_recextelectrode_01(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : -100,
            'tstop' : 100,
            'dt' : 2**-4,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,

        }

        electrodeParams = {
            'sigma' : 0.3,
            'x' : np.ones(11) * 100.,
            'y' : np.zeros(11),
            'z' : np.linspace(1000, 0, 11),
            'method' : 'pointsource'
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'delay' : 0.,
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'phase' : 0,
            'bias' : 0.,
            'record_current' : False
        }

        stick = LFPy.TemplateCell(**stickParams)
        synapse = LFPy.StimIntElectrode(stick, stick.get_closest_idx(0, 0, 1000),
                               **stimParams)
        electrode = LFPy.RecExtElectrode(**electrodeParams)
        stick.simulate(electrode, rec_imem=True)

        electrode1 = LFPy.RecExtElectrode(cell=stick, **electrodeParams)
        electrode1.calc_lfp()

        np.testing.assert_allclose(electrode.LFP, electrode1.LFP)
        self.assertTrue(stick.tvec.size == stick.imem.shape[1] ==
                        electrode.LFP.shape[1] == electrode1.LFP.shape[1] ==
                        int(stick.tstop/stick.dt)+1)

    def test_cell_with_recextelectrode_02(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 0.1,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,

        }

        electrodeParams = {
            'sigma' : 0.3,
            'x' : np.ones(11) * 100.,
            'y' : np.zeros(11),
            'z' : np.linspace(1000, 0, 11),
            'method' : 'pointsource'
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'delay' : 0.,
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'phase' : 0,
            'bias' : 0.,
            'record_current' : False
        }

        stick = LFPy.TemplateCell(**stickParams)
        synapse = LFPy.StimIntElectrode(stick, stick.get_closest_idx(0, 0, 1000),
                               **stimParams)
        electrode = LFPy.RecExtElectrode(**electrodeParams)
        stick.simulate(electrode, rec_imem=True)

        electrode1 = LFPy.RecExtElectrode(cell=stick, **electrodeParams)
        electrode1.calc_lfp()

        np.testing.assert_allclose(electrode.LFP, electrode1.LFP)
        self.assertTrue(stick.tvec.size == stick.imem.shape[1] ==
                        electrode.LFP.shape[1] == electrode1.LFP.shape[1] ==
                        int(stick.tstop/stick.dt)+1)

    def test_cell_with_recextelectrode_03(self):
        stickParams = {
            'morphology' : os.path.join(LFPy.__path__[0], 'test', 'stick.hoc'),
            'templatefile' : os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
            'templatename' : 'stick_template',
            'templateargs' : None,
            'cm' : 1,
            'Ra' : 150,
            'v_init' : -65,
            'passive' : True,
            'passive_parameters' : {'g_pas' : 1./30000, 'e_pas' : -65},
            'tstart' : 0,
            'tstop' : 100,
            'dt' : 0.2,
            'nsegs_method' : 'lambda_f',
            'lambda_f' : 100,

        }

        electrodeParams = {
            'sigma' : 0.3,
            'x' : np.ones(11) * 100.,
            'y' : np.zeros(11),
            'z' : np.linspace(1000, 0, 11),
            'method' : 'pointsource'
        }

        stimParams = {
            'pptype' : 'SinSyn',
            'delay' : 0.,
            'dur' : 1000.,
            'pkamp' : 1.,
            'freq' : 100.,
            'phase' : 0,
            'bias' : 0.,
            'record_current' : False
        }

        stick = LFPy.TemplateCell(**stickParams)
        synapse = LFPy.StimIntElectrode(stick, stick.get_closest_idx(0, 0, 1000),
                               **stimParams)
        electrode = LFPy.RecExtElectrode(**electrodeParams)
        stick.simulate(electrode, rec_imem=True)

        electrode1 = LFPy.RecExtElectrode(cell=stick, **electrodeParams)
        electrode1.calc_lfp()

        np.testing.assert_allclose(electrode.LFP, electrode1.LFP)
        self.assertTrue(stick.tvec.size == stick.imem.shape[1] ==
                        electrode.LFP.shape[1] == electrode1.LFP.shape[1] ==
                        int(stick.tstop/stick.dt)+1)

    def test_cell_distort_geometry_01(self):
        cell0 = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
        factors = [-0.2, 0.1, 0., 0.1, 0.2]
        nus = [-0.5, 0., 0.5]
        for factor in factors:
            for nu in nus:
                for axis in 'xyz':
                    cell1 = LFPy.TemplateCell(morphology=os.path.join(LFPy.__path__[0], 'test',
                                                  'ball_and_sticks_w_lists.hoc' ),
                        templatefile=os.path.join(LFPy.__path__[0], 'test', 'ball_and_stick_template.hoc'),
                        templatename='ball_and_stick_template',
                        templateargs=None,
                        )
                    cell1.distort_geometry(factor=factor, nu=nu, axis=axis)
                    for attr in ['start', 'mid', 'end']:
                        for ax in 'xyz'.replace(axis, ''):
                            np.testing.assert_allclose(getattr(cell0, ax+attr)*(1+factor*nu),
                                                       getattr(cell1, ax+attr))
                        np.testing.assert_allclose(getattr(cell0, axis+attr)*(1-factor),
                                                   getattr(cell1, axis+attr))


    ######## Functions used by tests: ##########################################

def stickSimulationTesttvec(**kwargs):
    stick = LFPy.TemplateCell(morphology = os.path.join(LFPy.__path__[0], 'test',
                                                'stick.hoc'), verbose=False,
                              templatefile=os.path.join(LFPy.__path__[0], 'test', 'stick_template.hoc'),
                              templatename='stick_template',
                              templateargs=None,
                              **kwargs)
    stick.simulate(rec_imem=False)
    return stick.tvec



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Example plot for LFPy: Single-synapse contribution to the potential through head

Copyright (C) 2017 Computational Neuroscience Group, NMBU.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
"""
import LFPy
import numpy as np
import matplotlib.pyplot as plt
from os.path import join

# four-sphere properties
radii = [79000., 80000., 85000., 90000.]
radii_name = ["Brain", "CSF", "Skull", "Scalp"]
sigmas = [0.3, 1.5, 0.015, 0.3]
rad_tol = 1e-2
somapos = np.array([0., 0., 78750.])

xlim = [-1000, 1000]
ylim = [radii[0]-500, radii[1] + 500]

# cell with simplified morphology
cellParameters = {
    'morphology' : join("morphologies", 'example_morphology.hoc'),
    'tstart' : 0.,
    'tstop' : 5.,
    'dt' : 2**-4,
    'v_init' : -65,
    'cm' : 1.,
    'Ra' : 150.,
    'passive' : True,
    'passive_parameters' : {'g_pas' : 1./3E4, 'e_pas' : -65.},
    'pt3d' : True,
}

cell = LFPy.Cell(**cellParameters)
synidx = cell.get_closest_idx(z=200)

synapseParameters = {
    'idx' : synidx,
    'e' : 0,                                # reversal potential
    'syntype' : 'Exp2Syn',                   # synapse type
    'tau1' : 0.1,                              # syn. rise time constant
    'tau2' : 1.,                              # syn. decay time constant
    'weight' : 0.002,                        # syn. weight
    'record_current' : True                 # syn. current record
}
synapse = LFPy.Synapse(cell, **synapseParameters)
synapse.set_spike_times(np.array([1.]))
cell.set_pos(z=somapos[2])
cell.simulate(rec_imem=True, rec_vmem=True, rec_current_dipole_moment=True)

# Setting up recording positions
elec_z = np.array([radii[0],
                  (radii[0] + radii[1]) / 2,
                   radii[1],
                   radii[1] + 500])

elec_x = np.zeros(elec_z.shape)
elec_y = np.zeros(elec_x.shape)
eeg_coords = np.array([elec_x.flatten(), elec_y.flatten(), elec_z.flatten()]).T

MD_4s = LFPy.FourSphereVolumeConductor(radii, sigmas, eeg_coords)
phi = MD_4s.calc_potential_from_multi_dipoles(cell) * 1e6  # from mV to nV

# Plotting results
plt.close('all')
fig = plt.figure(figsize=[8, 10])
fig.subplots_adjust(left=0.02, hspace=0.5, right=0.98, top=0.95)

ax = fig.add_subplot(111, aspect=1, frameon=False, xlim=xlim, ylim=ylim,
                     xticks=[], yticks=[])

# Plotting cell
for sec in LFPy.cell.neuron.h.allsec():
    idx = cell.get_idx(sec.name())
    l_cell, = ax.plot(np.r_[cell.xstart[idx], cell.xend[idx][-1]],
            np.r_[cell.zstart[idx], cell.zend[idx][-1]],
            color='r', zorder=3, lw=np.sqrt(np.average(cell.diam[idx])))

l_syn, = ax.plot(cell.xmid[synapseParameters['idx']],
                 cell.zmid[synapseParameters['idx']],
                 '*', c='orange', zorder=5)

# Plotting the layers of the head model
max_angle = np.abs(np.rad2deg(np.arcsin(xlim[0] / ylim[0])))
plot_angle = np.linspace(-max_angle, max_angle, 100)
for b_idx in range(len(radii)):
    x_ = radii[b_idx] * np.sin(np.deg2rad(plot_angle))
    z_ = radii[b_idx] * np.cos(np.deg2rad(plot_angle))
    l_curved, = ax.plot(x_, z_, ':', c="gray")

ax.text(xlim[0], radii[0] - 70, "Brain", va="top", ha="left", color="k")
ax.text(xlim[0], radii[0] + 70, "CSF", va="top", ha="left", color="k")
ax.text(xlim[0], radii[1] + 70, "Skull", va="top", ha="left", color="k")

ax.plot([-700, -200], [radii[0]-300, radii[0]-300], 'k', lw=2)
ax.text(-550, radii[0]-290, "500 $\mu$m")

# Plotting the potential at different positions
y_norm = 200
x_norm = 500
elec_clr = lambda elec_idx: plt.cm.viridis(elec_idx / (len(elec_z) - 1))
for elec_idx in range(len(elec_z)):
    l_elec, = ax.plot(elec_x[elec_idx], elec_z[elec_idx],
                      'o', c=elec_clr(elec_idx), clip_on=False)
    y_ = phi[elec_idx, :]
    norm_const = np.max(np.abs(y_))
    y_ = y_ * y_norm / norm_const + elec_z[elec_idx]
    x_ = cell.tvec / cell.tvec[-1] * x_norm + elec_x[elec_idx]
    ax.plot(x_, y_, c=elec_clr(elec_idx), clip_on=False)
    ax.plot([x_[-1] + 200, x_[-1] + 200],
            [elec_z[elec_idx], elec_z[elec_idx] - y_norm], c='k')
    scale_text = "{:2.1f} nV".format(norm_const)
    ax.text(x_[-1] + 250, elec_z[elec_idx] - y_norm / 2, scale_text)

ax.plot([x_[0], x_[-1]],
        [elec_z[1] - y_norm*1.1, elec_z[1] - y_norm*1.1],
        c='k', clip_on=False, lw=2)
ax.text((x_[0] + x_[-1])/2, elec_z[1] - y_norm * 1.2,
        "{} ms".format(cell.tvec[-1]),
        va="top", ha="center", clip_on=False)

ax_inset = fig.add_axes([0.7, 0.05, 0.25, 0.15], title="Synaptic current",
                        ylabel="nA", xlabel="Time (ms)")
l_isyn, = ax_inset.plot(cell.tvec, synapse.i, c='orange')

ax.legend([l_cell, l_curved, l_syn, l_isyn, l_elec],
          ["Cell", "Four-sphere boundary", "Synapse",
           "Synaptic current", "Electrode"],
          ncol=1, loc=(0.01, -0.1), frameon=False)

plt.savefig(join('example_potential_through_head.pdf'))
plt.show()
